//Test version 0.6
//

//--------------------------------------------------------------------------------------
// Defines
//--------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------
// Vectors, floats and etc.
//--------------------------------------------------------------------------------------
float4	BloomParameters;
float4	TempParameters;
float4  ScreenSize;


//Bloom

float MixFactor
<
	string UIName="Mix Factor";
	string UIWidget="Spinner";
	float UIMin=0.0;
	float UIMax=1000.0;
> = {0.33};
 
float BloomIntensity
<
	string UIName="Bloom Intensity";
	string UIWidget="Spinner";
	float UIMin=0.0;
	float UIMax=1000.0;
> = {2.20};

 
float BloomSaturation
<
	string UIName="Bloom Saturation";
	string UIWidget="Spinner";
	float UIMin=0.0;
	float UIMax=1000.0;
> = {1.25};


float BloomSmooth
<
	string UIName="Bloom Smooth";
	string UIWidget="Spinner";
	float UIMin=0.0;
	float UIMax=1000.0;
> = {8.20};

float Smoothness
<
	string UIName="Bloom Smoothness";
	string UIWidget="Spinner";
	float UIMin=0.0;
	float UIMax=1000.0;
> = {0.50};


//--------------------------------------------------------------------------------------
// Sampler Inputs, Textures
//--------------------------------------------------------------------------------------


//quad
struct VS_OUTPUT_POST
{
	float4 vpos  : POSITION;
	float2 txcoord0 : TEXCOORD0;
};
struct VS_INPUT_POST
{
	float3 pos  : POSITION;
	float2 txcoord0 : TEXCOORD0;
};




texture2D texBloom1;
texture2D texBloom2;
texture2D texBloom3;
texture2D texBloom4;
texture2D texBloom5;

sampler2D SamplerBloom1 = sampler_state
{
    Texture   = <texBloom1>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;//NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

sampler2D SamplerBloom2 = sampler_state
{
    Texture   = <texBloom2>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;//NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

sampler2D SamplerBloom3 = sampler_state
{
    Texture   = <texBloom3>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;//NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

sampler2D SamplerBloom4 = sampler_state
{
    Texture   = <texBloom4>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;//NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

sampler2D SamplerBloom5 = sampler_state
{
    Texture   = <texBloom5>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;//NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};



//--------------------------------------------------------------------------------------
// Vertex Shader Input
//--------------------------------------------------------------------------------------

VS_OUTPUT_POST VS_Bloom(VS_INPUT_POST IN)
{
	VS_OUTPUT_POST OUT;

	OUT.vpos=float4(IN.pos.x,IN.pos.y,IN.pos.z,1.0);

	OUT.txcoord0.xy=IN.txcoord0.xy+TempParameters.xy;//1.0/(bloomtexsize*2.0)

	return OUT;
}


//--------------------------------------------------------------------------------------
// Pixel Shader Effects
//--------------------------------------------------------------------------------------
float4 AdjustSaturation(float4 color, float saturation) 
{ 
  float grey = dot(color, float3(0.5, 0.5, 0.5)); 
  
  return lerp(grey, color, saturation); 
} 

//--------------------------------------------------------------------------------------
// Bloom
//--------------------------------------------------------------------------------------


float2 gaussOffsets[25]=
{
	float2(-0.707, 0.707),
	float2(-0.383, 0.924),
	float2(0.0, 1.0),
	float2(0.383, 0.924),
	float2(0.707, 0.707),
	
	float2(-0.924, 0.383),
	float2(-0.707, 0.707),
	float2(0.0, 1.0),
	float2(0.707, 0.707),
	float2(0.924, 0.383),
	
	float2(-1.0, 0.0),
	float2(-1.0, 0.0),
	float2(0.0, 0.0),
	float2(1.0, 0.0),
	float2(1.0, 0.0),
	
	float2(-0.924, -0.383),
	float2(-0.707, -0.707),
	float2(0.0, -1.0),
	float2(0.707, -0.707),
	float2(0.924, -0.383),
	
	float2(-0.707, -0.707),
	float2(-0.383, -0.924),
	float2(0.0, -1.0),
	float2(0.383, -0.924),
	float2(0.707, -0.707)
};



float4 PS_BloomPrePass(VS_OUTPUT_POST In) : COLOR 
{
        float4 bloom;
        float4 bloom2;
	float4 color = tex2D(SamplerBloom1, In.txcoord0);

        float2 screenfact=1.0;
	screenfact.y*=ScreenSize.z;
	screenfact.xy*=TempParameters.z*Smoothness;

        float4 bloomadd=color;

	for (int i=0; i<25; i++)
        {
          bloom.xy=In.txcoord0.xy;

          bloom.xy=gaussOffsets[i]*BloomParameters.z;
          bloom.xy=(bloom.xy*screenfact.xy)+In.txcoord0.xy;

          bloom2.xy=gaussOffsets[i];
          bloom2.xy=(bloom2.xy*screenfact.xy)+In.txcoord0.xy;

	  float4 temp1 = tex2D(SamplerBloom1, bloom.xy);
	  float4 temp2 = tex2D(SamplerBloom1, bloom2.xy);

	  float4 tempbloom = lerp(temp1, temp2, MixFactor);
	  bloomadd+=tempbloom;
		
	  color.xyz = max(color.xyz, tempbloom.xyz);
        }
        
	return color;
}


//--------------------------------------------------------------------------------------
// TextureBloom1
//--------------------------------------------------------------------------------------


float4 PS_TextureBloom1(VS_OUTPUT_POST In) : COLOR 
{
        float4 bloom;
	float4 color = tex2D(SamplerBloom1, In.txcoord0);

	for (int i=0; i<25; i++)
        {
          bloom += tex2D(SamplerBloom1, In.txcoord0.xy + ((gaussOffsets[i]*BloomParameters.z) / 99.006)* BloomSmooth) * 1.0;
        }

        bloom = AdjustSaturation(bloom, BloomSaturation) * BloomIntensity;
        
        bloom = (bloom / 35);
	return bloom;
}


//--------------------------------------------------------------------------------------
// TextureBloom2
//--------------------------------------------------------------------------------------


float4 PS_TextureBloom2(VS_OUTPUT_POST In) : COLOR 
{
        float4 bloom;
	float4 color = tex2D(SamplerBloom1, In.txcoord0);

	for (int i=0; i<25; i++)
        {
          bloom += tex2D(SamplerBloom1, In.txcoord0.xy + ((gaussOffsets[i]*BloomParameters.z) / 99.006)* BloomSmooth) * 1.0;
        }


        bloom = (bloom / 35);
	return bloom;
}

//--------------------------------------------------------------------------------------
// TextureBloom3
//--------------------------------------------------------------------------------------


float4 PS_TextureBloom3(VS_OUTPUT_POST In) : COLOR 
{
        float4 bloom;
	float4 color = tex2D(SamplerBloom1, In.txcoord0);


	for (int i=0; i<25; i++)
        {
          bloom += tex2D(SamplerBloom1, In.txcoord0.xy + ((gaussOffsets[i]*BloomParameters.z) / 99.006)* BloomSmooth) * 1.0;
        }

        bloom = (bloom / 35);
	return bloom;
}


//--------------------------------------------------------------------------------------
// BloomPostPass
//--------------------------------------------------------------------------------------

float4 PS_BloomPostPass(VS_OUTPUT_POST In) : COLOR
{
	float4 bloom;
	float4 temp;
	//v1
	bloom =tex2D(SamplerBloom1, In.txcoord0);
	bloom+=tex2D(SamplerBloom2, In.txcoord0);
	bloom+=tex2D(SamplerBloom3, In.txcoord0);
	bloom+=tex2D(SamplerBloom4, In.txcoord0);
	bloom+=tex2D(SamplerBloom5, In.txcoord0);
	bloom*=0.2;
	temp = bloom;

	//v2
	float4 bloom1=tex2D(SamplerBloom1, In.txcoord0);
	float4 bloom2=tex2D(SamplerBloom2, In.txcoord0);
	float4 bloom3=tex2D(SamplerBloom3, In.txcoord0);
	float4 bloom4=tex2D(SamplerBloom4, In.txcoord0);
	float4 bloom5=tex2D(SamplerBloom5, In.txcoord0);
	bloom=max(bloom1, bloom2);
	bloom=max(bloom, bloom3);
	bloom=max(bloom, bloom4);
	bloom=max(bloom, bloom5);

	bloom = lerp(temp, bloom, 0.3);

	//bloom =tex2D(SamplerBloom1, In.txcoord0);

	bloom.w=1.0;
	return bloom;
}


//--------------------------------------------------------------------------------------
// Compiler 1
//--------------------------------------------------------------------------------------

technique BloomPrePass
{	
	pass P0
	{
		VertexShader = compile vs_3_0 VS_Bloom();
		PixelShader  = compile ps_3_0 PS_BloomPrePass();
	        COLORWRITEENABLE=ALPHA|RED|GREEN|BLUE;
	        CullMode=NONE;
	        AlphaBlendEnable=FALSE;
	        AlphaTestEnable=FALSE;
	        SEPARATEALPHABLENDENABLE=FALSE;
	        FogEnable=FALSE;
	        SRGBWRITEENABLE=FALSE;
	}
}

technique BloomTexture2
{	
	pass P0
	{
		VertexShader = compile vs_3_0 VS_Bloom();
		PixelShader  = compile ps_3_0 PS_TextureBloom1();

	        COLORWRITEENABLE=ALPHA|RED|GREEN|BLUE;
	        CullMode=NONE;
	        AlphaBlendEnable=FALSE;
	        AlphaTestEnable=FALSE;
	        SEPARATEALPHABLENDENABLE=FALSE;
	        FogEnable=FALSE;
	        SRGBWRITEENABLE=FALSE;
	}
}

technique BloomTexture3
{	
	pass P0
	{
		VertexShader = compile vs_3_0 VS_Bloom();
		PixelShader  = compile ps_3_0 PS_TextureBloom2();

	        COLORWRITEENABLE=ALPHA|RED|GREEN|BLUE;
	        CullMode=NONE;
	        AlphaBlendEnable=FALSE;
	        AlphaTestEnable=FALSE;
	        SEPARATEALPHABLENDENABLE=FALSE;
	        FogEnable=FALSE;
	        SRGBWRITEENABLE=FALSE;
	}
}

technique BloomTexture4
{	
	pass P0
	{
		VertexShader = compile vs_3_0 VS_Bloom();
		PixelShader  = compile ps_3_0 PS_TextureBloom3();

	        COLORWRITEENABLE=ALPHA|RED|GREEN|BLUE;
	        CullMode=NONE;
	        AlphaBlendEnable=FALSE;
	        AlphaTestEnable=FALSE;
	        SEPARATEALPHABLENDENABLE=FALSE;
	        FogEnable=FALSE;
	        SRGBWRITEENABLE=FALSE;
	}
}

technique BloomPostPass
{
    pass p0
    {
	VertexShader = compile vs_3_0 VS_Bloom();
	PixelShader  = compile ps_3_0 PS_BloomPostPass();

	COLORWRITEENABLE=ALPHA|RED|GREEN|BLUE;
	CullMode=NONE;
	AlphaBlendEnable=FALSE;
	AlphaTestEnable=FALSE;
	SEPARATEALPHABLENDENABLE=FALSE;
	FogEnable=FALSE;
	SRGBWRITEENABLE=FALSE;
	}
}



